@@ -34,7 +34,7 @@ from .util import (
     prompt_for_touch)
 from ..driver_ccid import APDUError,  SW_APPLICATION_NOT_FOUND
 from ..util import TRANSPORT, derive_key, parse_uri, parse_b32_key
-from ..oath import OathController, SW
+from ..oath import OathController, SW, Credential
 
 click_touch_option = click.option(
     '-t', '--touch', is_flag=True,
@@ -120,21 +120,26 @@ def reset(ctx):
 @click.option(
     '-o', '--oath-type', type=click.Choice(['TOTP', 'HOTP']), default='TOTP',
     help='Time-based (TOTP) or counter-based'
-    ' (HOTP) credential (default is TOTP).')
+    ' (HOTP) credential.', show_default=True)
 @click.option(
-    '-d', '--digits', type=click.Choice(['6', '8']), default='6',
-    help='Number of digits in generated code (default is 6).')
+    '-d', '--digits', type=click.Choice(['6', '7', '8']), default='6',
+    help='Number of digits in generated code.', show_default=True)
 @click.option(
     '-a', '--algorithm', type=click.Choice(['SHA1', 'SHA256']),
     default='SHA1', help='Algorithm to use for '
-    'code generation (default is SHA1).')
+    'code generation.', show_default=True)
 @click.option(
     '-c', '--counter', type=click.INT, default=0,
     help='Initial counter value for HOTP credentials.')
+@click.option('-i', '--issuer', help='Issuer of the credential.')
+@click.option(
+    '-p', '--period', help='Number of seconds a TOTP code is valid.',
+    default=30, show_default=True)
 @click_touch_option
 @click_force_option
 @click.pass_context
-def add(ctx, key, name, oath_type, digits, touch, algorithm, counter, force):
+def add(ctx, key, name, issuer, period, oath_type, digits, touch, algorithm,
+        counter, force):
     """
     Add a new credential.
 
@@ -154,7 +159,8 @@ def add(ctx, key, name, oath_type, digits, touch, algorithm, counter, force):
     ensure_validated(ctx)
 
     _add_cred(
-        ctx, key, name, oath_type, digits, touch, algorithm, counter, force)
+        ctx, key, name, issuer, period, oath_type, digits, touch, algorithm,
+        counter, force)
 
 
 @oath.command()
@@ -182,23 +188,28 @@ def uri(ctx, uri, touch, force):
     ensure_validated(ctx)
 
     params = uri
-    name = params.get('name')
+    period, issuer, name = Credential.parse_long_name(params.get('name'))
     key = params.get('secret')
     key = parse_b32_key(key.upper())
     oath_type = params.get('type')
     digits = params.get('digits') or 6
     algo = params.get('algorithm') or 'SHA1'
     counter = params.get('counter') or 0
+    issuer = params.get('issuer') or issuer
+    period = params.get('period') or period
 
     # Steam is a special case where we allow the otpauth
     # URI to contain a 'digits' value of '5'.
     if digits == 5 and name.startswith('Steam:'):
         digits = 6
 
-    _add_cred(ctx, key, name, oath_type, digits, touch, algo, counter, force)
+    _add_cred(ctx, key=key, name=name, issuer=issuer, period=period,
+              oath_type=oath_type, digits=digits, touch=touch, algo=algo,
+              counter=counter, force=force)
 
 
-def _add_cred(ctx, key, name, oath_type, digits, touch, algo, counter, force):
+def _add_cred(ctx, key, name, issuer, period, oath_type, digits, touch, algo,
+              counter, force):
 
     controller = ctx.obj['controller']
 
@@ -214,7 +225,9 @@ def _add_cred(ctx, key, name, oath_type, digits, touch, algo, counter, force):
     if counter and not oath_type == 'hotp':
         ctx.fail('Counter only supported for HOTP credentials.')
 
-    if not force and any(cred.name == name for cred in controller.list()):
+    long_name = Credential.build_long_name(int(period), issuer, name)
+    if not force and any(
+            cred.long_name == long_name for cred in controller.list()):
         click.confirm(
             'A credential called {} already exists on the device.'
             ' Do you want to overwrite it?'.format(name), abort=True)
@@ -231,8 +244,9 @@ def _add_cred(ctx, key, name, oath_type, digits, touch, algo, counter, force):
 
     try:
         controller.put(
-            key, name, oath_type=oath_type, digits=int(digits),
-            require_touch=touch, algo=algo, counter=int(counter))
+            key, name, issuer=issuer, period=int(period), oath_type=oath_type,
+            digits=int(digits), require_touch=touch, algo=algo,
+            counter=int(counter))
     except APDUError as e:
         if e.sw == SW.NO_SPACE:
             ctx.fail('No space left on device.')
@@ -248,7 +262,8 @@ def _add_cred(ctx, key, name, oath_type, digits, touch, algo, counter, force):
 @click.pass_context
 @click.option('-o', '--oath-type', is_flag=True, help='Display the OATH type.')
 @click.option('-a', '--algorithm', is_flag=True, help='Display the algorithm.')
-def list(ctx, show_hidden, oath_type, algorithm):
+@click.option('-p', '--period', is_flag=True, help='Display the period.')
+def list(ctx, show_hidden, oath_type, algorithm, period):
     """
     List all credentials.
 
@@ -259,13 +274,16 @@ def list(ctx, show_hidden, oath_type, algorithm):
     creds = [c for c in controller.list()]
     creds.sort()
     for cred in creds:
-        if cred.hidden and not show_hidden:
+        if cred.is_hidden() and not show_hidden:
             continue
-        click.echo(cred.name, nl=False)
+        full_name = cred.issuer + ':' + cred.name if cred.issuer else cred.name
+        click.echo(full_name, nl=False)
         if oath_type:
             click.echo(', {}'.format(cred.oath_type), nl=False)
         if algorithm:
             click.echo(', {}'.format(cred.algo), nl=False)
+        if period:
+            click.echo(', {}'.format(cred.period), nl=False)
         click.echo()
 
 
@@ -289,7 +307,7 @@ Touch and HOTP credentials require a single match to be triggered.
 
     # Remove hidden creds
     if not show_hidden:
-        creds = [c for c in creds if not c.hidden]
+        creds = [c for c in creds if not c.is_hidden()]
     if query:
         hits = _search(creds, query)
         if len(hits) == 1:
@@ -305,42 +323,49 @@ Touch and HOTP credentials require a single match to be triggered.
                 hotp_touch_timer.cancel()
             else:
                 cred = controller.calculate(cred)
-            click.echo('{} {}'.format(cred.name, cred.code))
+            if cred.issuer:
+                click.echo('{}:{} {}'.format(cred.issuer, cred.name, cred.code))
+            else:
+                click.echo('{} {}'.format(cred.name, cred.code))
             ctx.exit()
         creds = hits
 
-    longest = max(len(cred.name) for cred in creds) if creds else 0
+    longest = max(len('{}:{}'.format(
+        cred.issuer, cred.name)) for cred in creds) if creds else 0
     format_str = '{:<%d}  {:>10}' % longest
 
     creds.sort()
 
     for cred in creds:
+        full_name = cred.issuer + ':' + cred.name if cred.issuer else cred.name
         if cred.oath_type == 'totp':
-            click.echo(format_str.format(cred.name, cred.code))
+            click.echo(format_str.format(full_name, cred.code))
         if cred.touch:
-            click.echo(format_str.format(cred.name, '[Touch Credential]'))
+            click.echo(format_str.format(full_name, '[Touch Credential]'))
         if cred.oath_type == 'hotp':
-            click.echo(format_str.format(cred.name, '[HOTP Credential]'))
+            click.echo(format_str.format(full_name, '[HOTP Credential]'))
 
 
 @oath.command()
 @click.pass_context
 @click.argument('query')
-def remove(ctx, query):
+def delete(ctx, query):
     """
-    Remove a credential.
+    Delete a credential.
 
-    Remove a credential from the device. \
-Provide a query string to match the credential to remove.
+    Delete a credential from the device. \
+Provide a query string to match the credential to delete.
     """
 
     ensure_validated(ctx)
     controller = ctx.obj['controller']
     creds = controller.list()
     hits = _search(creds, query)
-    if len(hits) == 1:
+    if len(hits) == 0:
+        click.echo('No matches, nothing to be done.')
+    elif len(hits) == 1:
         controller.delete(hits[0])
-        click.echo('Removed {}.'.format(hits[0].name))
+        click.echo('Deleted {}.'.format(hits[0].long_name))
     else:
         click.echo('To many matches, please specify the query.')
 
@@ -392,9 +417,9 @@ def _validate(ctx, password):
 def _search(creds, query):
     hits = []
     for c in creds:
-        if c.name == query:
+        if c.long_name == query:
             return [c]
-        if query.lower() in c.name.lower():
+        if query.lower() in c.long_name.lower():
             hits.append(c)
     return hits
 
