@@ -63,22 +63,29 @@ def _failed_to_write_msg(ctx):
 @click.group()
 @click.pass_context
 @click_skip_on_help
-@click.option('--access-code', required=False, metavar='HEX',
-              help='If your YubiKey is write-protected using an access code, '
-              'you will need to specify it here for any operation that writes '
-              'to the device. Set to empty to use a prompt for input.')
+@click.option(
+    '--access-code', required=False, metavar='HEX',
+    help='A 6 byte access code. Set to empty to use a prompt for input.')
 def slot(ctx, access_code):
     """
     Manage YubiKey Slots.
 
     The YubiKey provides two keyboard-based slots which can each be configured
     with a credential. Several credential types are supported.
+
+    A slot configuration may be write-protected with an access code. This
+    prevents the configuration to be overwritten without the access code
+    provided. Mode switching the device is not possible when a slot is
+    configured with an access code.
     """
 
     if access_code is not None:
         if access_code == '':
             access_code = click.prompt('Enter access code', show_default=False)
-        access_code = a2b_hex(access_code)
+        try:
+            access_code = a2b_hex(access_code)
+        except TypeError as e:
+            raise ValueError(e)
         if len(access_code) != 6:
             raise ValueError('Must be exactly 6 bytes.')
     ctx.obj['dev'].driver.access_code = access_code
@@ -408,10 +415,13 @@ def hotp(ctx, slot, key, digits, counter, no_enter, force):
 @click_force_option
 @click.pass_context
 @click.option(
-    '--enter/--no-enter',
-    # default=True,
+    '--enter/--no-enter', default=True, show_default=True,
     help="Should send 'Enter' keystroke after slot output.")
-def settings(ctx, slot, enter, force):
+@click.option(
+    '-p', '--pacing', type=click.Choice(['0', '20', '40', '60']),
+    default='0', show_default=True, help='Throttle output speed by '
+    'adding a delay (in ms) between characters emitted.')
+def settings(ctx, slot, enter, pacing, force):
     """
     Update the settings for a slot.
 
@@ -425,8 +435,12 @@ def settings(ctx, slot, enter, force):
         'Update the settings for slot {}? '
         'All existing settings will be overwritten.'.format(slot), abort=True)
     click.echo('Updating settings for slot {}...'.format(slot))
+
+    if pacing is not None:
+        pacing = int(pacing)
+
     try:
-        dev.driver.update_settings(slot, enter)
+        dev.driver.update_settings(slot, enter=enter, pacing=pacing)
     except YkpersError:
         _failed_to_write_msg(ctx)
 
