@@ -29,7 +29,9 @@
 from __future__ import absolute_import
 from enum import IntEnum, unique
 from .driver_ccid import APDUError, SW_OK, SW_APPLICATION_NOT_FOUND
-from .util import AID, Tlv, parse_tlvs
+from .util import (
+    AID, Tlv, parse_tlvs,
+    ensure_not_cve201715361_vulnerable_firmware_version)
 from cryptography import x509
 from cryptography.utils import int_to_bytes, int_from_bytes
 from cryptography.hazmat.primitives import hashes
@@ -96,6 +98,16 @@ class ALGO(IntEnum):
             return cls.ECCP384
         raise ValueError('Unsupported algorithm!')
 
+    @classmethod
+    def is_rsa(cls, algorithm_int):
+        # Implemented as "not not RSA" to reduce risk of false negatives if
+        # more algorithms are added
+        return not (
+            algorithm_int == cls.TDES
+            or algorithm_int == cls.ECCP256
+            or algorithm_int == cls.ECCP384
+        )
+
 
 @unique
 class SLOT(IntEnum):
@@ -640,6 +652,10 @@ class PivController(object):
 
     def generate_key(self, slot, algorithm, pin_policy=PIN_POLICY.DEFAULT,
                      touch_policy=TOUCH_POLICY.DEFAULT):
+
+        if ALGO.is_rsa(algorithm):
+            ensure_not_cve201715361_vulnerable_firmware_version(self.version)
+
         data = Tlv(TAG.ALGO, six.int2byte(algorithm))
         if pin_policy:
             data += Tlv(TAG.PIN_POLICY, six.int2byte(pin_policy))
